package com.jdauny.client;
import java.io.*;
import java.util.Vector;
import com.jdauny.*;
public class DaunySocket extends Socket_Com
{
  boolean m_bTransmitting,m_bBusy;
  boolean m_bLogged_in = false;
  boolean m_bIOrights = true;
  DaunyOutput m_master;
  UserDat m_userdat;
  String m_sSavefile = "user.dat";
  int m_iActual;
  int m_iUpdate;
  int m_iStartGet;
  int m_iEndGet;

  int m_links_to_update[];

  /** Konstruktor
 @param master Oberflche welche auf DaunySocket aufsetzt
 @param iorights Soll versucht werden ein UserDat File anzulegen ?
 */
  public DaunySocket(DaunyOutput master, boolean iorights)
  {
    m_master = master;
    m_bIOrights = iorights;
    if (!ReadUserData())
    {
      m_userdat = new UserDat();
      m_iStartGet = 0;
      m_iEndGet = 0;
      Reset(false);
    }
  }

  /** Setzt Namen des saveFile und versucht Daten zu lesen
 @param saveFile Name des SaveFile
 */
  public void setSaveFile(String saveFile)
  {
    m_sSavefile = saveFile;
    m_bIOrights = true;
  }


  /** Setzt DaunySocket in Startzustand zurck
 @param which Gibt an ob auch die Link Infos gelscht werden sollen
 */
  public void Reset(boolean which)
  {
    m_iActual = 0;
    m_iUpdate = 0;
    m_bTransmitting = false;
    m_bBusy = false;
    if (which)
    {
      // Userdate links lschen
      m_userdat.emptyLinks();
    }
  }

  /** Checkt um DaunySocket Befehle schicken kann*/
  public boolean isSendAble()
  {
    return ((m_bLogged_in || m_userdat.isProxy()) && !m_bTransmitting && !m_bBusy );
  }

  /** berprft ob der Client beim Server eingeloggt ist*/
  public boolean isLogged_in()
  {
    return m_bLogged_in;
  }

  /** Versucht Verbindung mit Server aufzumachen und sich dann einzuloggen */
  protected void Login()
  {
    if (!m_bConnected)
    {
      m_bCSend = true;
      if (Open(m_userdat.getServer(),m_userdat.getPort()))
      {
        if (m_bCSend)
        {
          Vector param = new Vector();
          param.add(m_userdat.getUser());
          param.add(m_userdat.getPwd());
          DaunyMessage newMessage = new DaunyMessage(param);
          newMessage.setType(newMessage.MSG_LOGIN);
          Send(newMessage);
        }
        else
        {
          String message = "LOGIN "+m_userdat.getUser()+" "+m_userdat.getPwd();
//						System.out.println(message);
          Send(message);
        }
        m_master.UpdateSend(0,"Login "+m_userdat.getUser()+" ***");
      }
      else
      {
        m_master.setOutput(100,"Not connected!\nNo server was found \nat "+m_userdat.getServer()+"\n on portnumber "
                           +m_userdat.getPort()+".");//" to "+(m_userdat.getPort()+10)+".");
      }
    }
    else
    {
      m_master.setOutput(6,"You are already connected !");
    }
  }

  /** Logout vom Server*/
  public void requestLogout()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_LOGOUT);
        Send(newMessage);
      }
      else
      {
        Send("Logout");
      }
      m_master.UpdateSend(0,"Logout");
    }
  }

  /** ndern des Passwortes
 @param newpwd neues Pawort
 */
  public void requestChangePWD(String newpwd)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(m_userdat.getPwd());
        parameter.add(newpwd);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_CHANGEPWD);
        Send(newMessage);
      }
      else
      {
        Send("CHANGEPASSW "+newpwd);
      }
      m_master.UpdateSend(0,"ChangePwd");
    }
  }

  /** Fragt die Server Version ab*/
  public void requestServerVersion()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_VERSION);
        Send(newMessage);
      }
      else
      {
        String message = "VER";
        Send(message);
      }
      m_master.UpdateSend(0,"VER");
    }
  }

  /** Fragt Server Status ab*/
  public void requestServerStatus()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_SERVERSTATUS);
        Send(newMessage);
      }
      else
      {
        String message = "VAR";
        Send(message);
      }
      m_master.UpdateSend(0,"VAR");
    }
  }

  /** Fragt Users ab, die gerade im System eingeloggt sind*/
  public void requestUsers()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_USERS);
        Send(newMessage);
      }
      else
      {
        String message = "USERS";
        Send(message);
      }
      m_master.UpdateSend(0,"USERS");
    }
  }

  /** Schickt Liste von LinkIDs der updzudatenden Links*/
  public void requestUpdate()
  {
    if (isSendAble())
    {
      m_master.setBusy(true);
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        Link thislink;
        for(int i=0;i<m_userdat.numberofLinks();i++)
        {
          thislink = (Link) m_userdat.getLink(i);
          if(thislink.getType() == -1)
            parameter.add(new Integer(thislink.getID()));
        }
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_UPDATE);
        Send(newMessage);
      }
      else
      {
        m_links_to_update = new int[m_userdat.numberofLinks()];
        Link link;
        int iType;
        int link_index = 0;
        int userdat_index = 0;
        while(userdat_index < m_userdat.numberofLinks())
        {
          link = m_userdat.getLink(userdat_index);
          iType = link.getType();

          if(iType==-1)// && (link.getState().equals("downloading") || link.getState().equals("queing")))
          {
            m_links_to_update[link_index++] = link.getID();
          }
          userdat_index++;
        }
        if(link_index < m_links_to_update.length)
          m_links_to_update[link_index] = -1;
        SendUpdate();
      }
      m_master.UpdateSend(0,"Update");
    }
  }

  public void requestUpdate(int[] list_to_update)
  {
    if (isSendAble())
    {
      if(m_bCSend)
      {
        return;
      }
      m_master.setBusy(true);
      m_links_to_update = new int[list_to_update.length];
      for(int index=0;index<list_to_update.length;index++)
        m_links_to_update[index] = ((Link)m_userdat.getLink(list_to_update[index])).getID();
      SendUpdate();
    }
  }

  /** Schickte Liste von Link-Klassen zum downloaden*/
  public void requestGet()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Link thislink;
        boolean take = false;
        Vector parameter = new Vector();
        for(int i=0;i<m_userdat.numberofLinks();i++)
        {
          thislink = (Link) m_userdat.getLink(i);
          if (thislink.getType() == 0 && !take)
          {
            take = true;
            m_iStartGet = i;
          }
          if (take)
          {
            parameter.add(thislink);
          }
        }
        if (!parameter.isEmpty())
        {
          m_iEndGet = m_userdat.numberofLinks()-1;
          DaunyMessage newMessage = new DaunyMessage(parameter);
          newMessage.setType(newMessage.MSG_GET);
          Send(newMessage);
          m_master.UpdateSend(0,"Get");
        }
      }
      else
      {
        m_iActual = 0;
        m_bTransmitting = true;
        //			System.out.println("-> Send: ");
        SendGet();
      }
    }
  }


  /** Sendet Anforderung an Server um Link nochmals herunterzuladen
 @param which ID des Links
 */
  public void requestReget(int which)
  {
    if (isSendAble())
    {
      Link thislink = (Link) m_userdat.getLink(which);
      if (thislink.getType()!=0)
      {
//				thislink.setType(1);
//				thislink.Update(thislink.getID(),0,"reget","");
        if (m_bCSend)
        {
          Vector parameter = new Vector();
          parameter.add(new Integer(thislink.getID()));
          DaunyMessage newMessage = new DaunyMessage(parameter);
          newMessage.setType(newMessage.MSG_REGET);
          Send(newMessage);
        }
        else
        {
          m_iActual = which;
          Send("Reget "+thislink.getID());
        }
        m_master.UpdateSend(0,"Reget "+thislink.getID());
//				return true;
      }
//			else return false;
    }

  }


  /** Sendet Anforderung an Server um meherere Link nochmals herunterzuladen
 @param which Vector mit IDs der Links
 */
  public void requestReget(Vector which)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.addAll(which);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_REGET);
        Send(newMessage);
      }
    }

  }


  /** Veranlat den Server eine Info Dateianzulegen
 @param filename Name des zu anzulegenden Files
 @param info Information die im File gespeichert werden soll
 */
  public void requestStore(String filename, String info)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(filename);
        parameter.add(getDir());
        parameter.add(info);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_STORE);
        Send(newMessage);
      }
      else
      {
        String message = "STORE \""+getDir()+"\" \""+filename+"\" \""+info+"\"";
        Send(message);
      }
      m_master.UpdateSend(0,"Store "+filename);
    }
  }

  /** Fragt Informationen bzgl. des Systems ab, auf dem der Server luft */
  public void requestSysInfo()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_SYSINFO);
        Send(newMessage);
      }
      else
      {
        Send("SYSINFO");
      }
      m_master.UpdateSend(0,"SysInfo");
    }
  }


  /** Sendet den Stop Befehl fr einen Link
 @param pos Position des zu stoppenden Links in UserDat
 */
  public void requestStop(int pos)
  {
    if (isSendAble())
    {
      int link_id = m_userdat.getLink(pos).getID();
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(new Integer(link_id));
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_STOP);
        Send(newMessage);
      }
      else
      {
        Send("STOP "+link_id);
      }
      //m_master.UpdateSend(0,command);
    }
  }

  /** Sendet den Stop Befehl fr die angebenen Links
 @param pos Vector mit den zu stoppenden Links
 */
  public void requestStop(Vector ids)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.addAll(ids);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_STOP);
        Send(newMessage);
      }
      else
      {
        //Send(command);
      }
      //m_master.UpdateSend(0,command);
    }
  }

  /** bertrgt einen Befehl an den Server */
  public void requestSend(String command)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(command);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_UNKNOWN);
        Send(newMessage);
      }
      else
      {
        Send(command);
      }
      m_master.UpdateSend(0,command);
    }
  }

  public void requestRemoveFile(int pos)
  {
    if (isSendAble())
    {
      Link this_link = m_userdat.getLink(pos);
      if (m_bCSend)
      {
        // not implemented
      }
      else
      {
        m_iActual = this_link.getID();
        String command = "RMFILEID "+this_link.getID();
        System.out.println(command);
        Send(command);
      }
    }
  }

  void OnClose()
  {
    if (m_bLogged_in)
    {
      m_master.setOutput(5,"Form Server disconneted!");
    }
    m_bLogged_in = false;

  }

  /** Zuteilen der empfangenen Nachrichten zu den einzelnen Funktionen - ClassMode
 @param message empfangene Nachricht
 */
  void OnReceive(com.jdauny.DaunyMessage message)
  {
    try
    {
      switch (message.getType())
      {
        case (0): Receive_Login(message); break;
        case (1): Receive_Logout(message); break;
        case (2): Receive_ChangePWD(message); break;
        case (3): Receive_Version(message); break;
        case (4): Receive_ServerStatus(message); break;
        case (5): Receive_Users(message); break;
        case (6): Receive_Get(message); break;
        case (7): Receive_Update(message); break;
        case (8): Receive_Reget(message); break;
        case (9): Receive_Stop(message); break;
        case (10): Receive_Store(message); break;
        case (11): Receive_SysInfo(message); break;
        case (12): Receive_LifeBeep(message); break;
          // *** ADMIN ***
        case (1000) : Receive_AddUser(message);break;
        case (1001) : Receive_DelUser(message);break;
        case (1002) : Receive_GetDownloads(message);break;
        case (1003) : Receive_DelDownload(message);break;
        case (1004) : Receive_SetMaxDownloads(message);break;
        case (1005) : Receive_GetMaxDownloads(message);break;
        case (1006) : Receive_GetUsers(message);break;
        case (1007) : Receive_UserEnable(message);break;
          // *** ADMIN ENDE ***
        default: //Receive_Unknown(message);
      }
//		System.out.println("ID: "+message.getID()+"\nSuxcess: "+message.getSuxcess()+"\nParameters: "+message.getParameters());
//		System.out.println("Size of Vector:"+message.getParameters().size());
//		System.out.println("##############");
      message = null;
    }
    catch(Exception e)
    {
      System.out.println("ERROR IN ON_RECEIVE");
      e.printStackTrace();
    }
  }

  /** Funktion Login
 @param message Login Nachricht
 */
  void Receive_Login(com.jdauny.DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      boolean admin = ((Boolean) message.getParameters().get(0)).booleanValue();
      m_bLogged_in = true;
      if (admin)
      {
        m_master.setOutput(10000,null);
      }
      else
      {
        m_master.setOutput(0,new Integer(0));
      }
    }
    else
    {
      m_bLogged_in = false;
      int error;
      try
      {
        error = ((Integer) message.getParameters().get(0)).intValue();
      }
      catch(Exception e)
      {
        error = 0;
      }
      m_master.setOutput(0,new Integer(error+1));
      Close();
    }
  }

  /** Funktion Logout
 @param message Logout Nachricht
 */
  void Receive_Logout(com.jdauny.DaunyMessage message)
  {
    Close();
    m_bLogged_in = false;
    //	m_master.setOutput(5,"Form Server disconneted !");
  }

  /** Funktion Users
 @param message Users Nachricht
 */
  void Receive_Users(com.jdauny.DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      String result = "";
      java.util.Enumeration users_online = message.getParameters().elements();
      while (users_online.hasMoreElements())
      {
        result += " "+(String) users_online.nextElement();
      }
      m_master.setOutput(2,result);
    }
  }

  /** Funktion Version
 @param message Version Nachricht
 */
  void Receive_Version(com.jdauny.DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      String result="";
      java.util.Enumeration paramString = message.getParameters().elements();
      while(paramString.hasMoreElements())
      {
        result += (String) paramString.nextElement() + "\n";
      }
      m_master.setOutput(3,result);
    }
  }

  /** Funktion ServerStatus
 @param message ServerStatus Nachricht
 */
  void Receive_ServerStatus(com.jdauny.DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      String result;
      result = "Downloads: "+(String)message.getParameters().get(0)+"\n"+
               "Queued: "+(String)message.getParameters().get(1)+"\n"+
               "Finished: "+(String)message.getParameters().get(2)+"\n"+
               "Users online: "+(String)message.getParameters().get(3);
      m_master.setOutput(1,result);
    }
  }

  /** Funktion ChangePWD
 @param message ChangePWD Nachricht
 */
  void Receive_ChangePWD(DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      m_master.setOutput(7,new Boolean(true));
      m_userdat.setPwd((String)message.getParameters().get(0));
    }
    else
    {
      m_master.setOutput(7,new Boolean(false));
    }
  }

  /** Funktion Update
 @param message Update Nachricht
 */
  void Receive_Update(DaunyMessage message)
  {
    if (message.getSuxcess())
    {

      Vector parameter = message.getParameters();
      m_userdat.updateLinks(parameter);
        /*		Link thislink;
   int actnumber = 0;
   for(int i=0;i<parameter.size();i++)
   {
    thislink = (Link)parameter.get(i);
    while(!m_userdat.getLink(actnumber).HasID(thislink.getID()))
    {
     actnumber++;
     if (actnumber>=m_userdat.numberofLinks())
     {
      i = parameter.size();
      break;
     }
    }
    if (i!=parameter.size())
    {
     m_userdat.replaceLink(actnumber,thislink);
    }
   }
      // Update Table in GUI
   m_master.UpdateReceive(2,null);
        */
    }
  }


  /** Funktion Get
 @param message Get Nachricht
 */
  void Receive_Get(DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      System.out.println("RECEIVE_GET");
      Vector parameter = message.getParameters();
      Link actLink;
      m_userdat.updateLinks(m_iStartGet,parameter);

      // Funkt des wirklich - vielleicht gibt es eine bessere Lsung
                    /*    m_userdat.removeLinks(m_iStartGet,m_iEndGet);
   Link thislink;
   for(int i=0;i<parameter.size();i++)
   {
    ((Link)parameter.get(i)).setType(-1);
   }
   m_userdat.addLinkVector(m_iStartGet,parameter);

   Vector updateTable = new Vector();
   updateTable.add(new Integer(m_iStartGet));
   updateTable.addAll(parameter);
   m_master.UpdateReceive(1,updateTable);
                    */
    }
  }

  /** Funktion Reget
 @param message Reget Nachricht
 */
  void Receive_Reget(DaunyMessage message)
  {
    Receive_Update(message);
  }

  /** Funktion Reget
 @param message Reget Nachricht
 */
  void Receive_Stop(DaunyMessage message)
  {
    Receive_Update(message);
  }

  /** Funktion Store
 @param message Store Nachricht
 */
  void Receive_Store(DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      // Datei erfolgreich angelegt
    }
    else
    {
      // Fehler beim anlegen der Datei
    }
  }

  /** Funktion SysInfo
 @param message SysInfo Nachricht
 */
  void Receive_SysInfo(DaunyMessage message)
  {
    if (message.getSuxcess())
    {
      String result = "";
      Vector parameter = message.getParameters();
      for(int i=0;i<parameter.size();i++)
        result += (String) parameter.get(i) + "\n";
      m_master.setOutput(4,result);
    }

  }

  /** Funktion LifeBeep
 @param message LifeBeep Nachricht
 */
  void Receive_LifeBeep(DaunyMessage message)
  {
    message.setSuxcess(true);
    // Client ist noch da
    Send(message);
  }

  /** Diese Methode wird aufgerufen wenn eine neue Nachricht empfangen wurde - TextMode
 @param message Die empfangene Nachricht
 */
  void OnReceive(String message)
  {
    try
    {
      m_master.UpdateReceive(0,message);
      if (!m_bLogged_in)
      {
        if (message.equals("LOGIN OK"))
        {
          m_bLogged_in = true;
          m_master.setOutput(0,new Integer(0));
        }
      }
      else
      {
        if (((String)getParameter(1," ",message).get(1)).equals("ERROR"))
        {
          m_master.setOutput(100,message);
          m_bTransmitting = false;
          m_bBusy = false;
        }
        else
        {

          //BYE
          if (message.startsWith("Bye"))
          {
            Close();
            m_bLogged_in = false;
            //	m_master.setOutput(5,"Form Server disconneted !");
          }

          // VAR
          if (message.startsWith("VAR"))
          {
            String result;
            result = "Downloads: "+(String)getParameter(1," ",message).get(1)+"\n"+
                     "Queued: "+(String)getParameter(2," ",message).get(1)+"\n"+
                     "Finished: "+(String)getParameter(3," ",message).get(1)+"\n"+
                     "Users online: "+(String)getParameter(4," ",message).get(1);
            m_master.setOutput(1,result);
          }

          if (message.startsWith("USERS"))
          {
            String result = "";
            int i = 1;
            while (((Boolean) getParameter(i," ",message).get(0)).booleanValue())
            {
              result = result +(String) getParameter(i," ",message).get(1)+" ";
              i++;
            }
            m_master.setOutput(2,result);
          }

          if (message.startsWith("VER"))
          {
            String result = "";
            m_master.setOutput(3,message.substring(4));
          }

          if (message.startsWith("SYSINFO"))
          {
            String result = "";
            result = "Files: "+(String) getParameter(1," ",message).get(1);
            result = result+"\n"+"Stored: "+(String) getParameter(2," ",message).get(1);
            result = result+"\n"+"Space: "+(String) getParameter(4," ",message).get(1);
            result = result+"\n"+"System: "+(String) getParameter(3," ",message).get(1);
            m_master.setOutput(4,result);
          }

          // Status
          if (message.startsWith("STATUS"))
          {
            int id,size;
            String sError,sState;
            int download_progress;
            id = Integer.parseInt((String) getParameter(1," ",message).get(1));
            size = Integer.parseInt((String) getParameter(4," ",message).get(1));
            sState = (String) getParameter(5," ",message).get(1);
            sError = (String) getParameter(6," ",message).get(1);
            download_progress = Integer.parseInt((String) getParameter(8," ",message).get(1));
            Link thislink = m_userdat.getLinkWithId(id);
            if (thislink != null)//.getID()==id)
            {
              thislink.Update(id,size,sState,sError);
              thislink.setProgress(download_progress);
              //					Updaten der Anzeige
  /*					Vector updateTable = new Vector(2);
                                          updateTable.add(new Integer(m_iUpdate));
                                          updateTable.add(thislink);
                                          m_master.UpdateReceive(1,updateTable);
  */
            }
            m_iUpdate++;
            SendUpdate();
          }

          // Get
          if (message.startsWith("GET"))
          {
            //		System.out.println("->Get");
            if(m_bTransmitting)
            {
              //			System.out.println("   ->Massensendung");
              String parameter;
              parameter =  (String) getParameter(1," ",message).get(1);
              int iID = Integer.parseInt(parameter);
              String sState = (String) getParameter(2," ",message).get(1);
              String sError = (String) getParameter(3," ",message).get(1);
              Link thislink = m_userdat.getLink(m_iActual);
              thislink.Update(iID,0,sState,sError);
              thislink.setType(-1);
              //					Updaten der Anzeige
  /*
                                          Vector updateTable = new Vector();
                                          updateTable.add(new Integer(m_iActual));
                                          updateTable.add(thislink);
                                          m_master.UpdateReceive(1,updateTable);
  */
              m_iActual++;
              SendGet();
            }

          }
          if (message.startsWith("REGET"))
          {
            System.out.println("Receive: "+message);
            Link this_link = m_userdat.getLink(m_iActual);
            int new_id = Integer.parseInt((String) getParameter(1," ",message).get(1));
            this_link.Update(new_id,0,"reget","");
            this_link.setProgress(0f);
          }
          if (message.startsWith("STOP"))
          {
            System.out.println("Receive: "+message);
            String result = (String) getParameter(1," ",message).get(1);
            if(result.equals("OK"))
            {

            }
            else
            {
              System.out.println("Error");
            }
          }
          if (message.startsWith("RMFILEID"))
          {
            System.out.println("Receive: "+message);
            String result = (String) getParameter(1," ",message).get(1);
            if(result.equals("OK"))
            {
              m_userdat.removeLinks(m_iActual,m_iActual);
            }
            else
            {
              System.out.println("Error");
            }
          }
          m_bBusy = false;
        }
      }
    }
    catch(Exception E)
    {
//			System.out.println("Error in Receive: "+E.toString());
//			System.out.println("message where error occured: "+message);
      m_iUpdate = 0;
      m_bBusy = false;
      m_bTransmitting = false;
    }
  }


  /** Massensendung von Links - kann nur von Receive aufgerufen werden
   */
  void SendGet()
  {
    //	System.out.println("->SendGet");
    String sServer,sUser,sPwd,sMessage;
    if(m_bConnected || m_userdat.isProxy())
    {
      // Link bertragen
      while(m_iActual < m_userdat.numberofLinks())
      {
        String sUrl,sDir;
        int iRetries,iSame,iPriority,iType;
        Link link = m_userdat.getLink(m_iActual);
        iType = link.getType();
        if(iType>=0)
        {
          switch(iType)
          {
            case 0:
              sUrl = link.getUrl();
              iRetries = link.getRetries();
              iSame = link.getSame();
              sDir = link.getDir();
//							System.out.println(sDir);
              iPriority = link.getPriority();
//							if (!sDir.equals(""))
//							{
              sDir="\""+sDir+"\"";
//							}
              sMessage ="GET "+"\""+sUrl+"\" "+iRetries+" "+" "+iPriority+" "+sDir+" "+iSame;
              break;
            case 1:
              sMessage = "REGET "+link.getID();
              break;
            default:
              sUrl = link.getUrl();
            iRetries = link.getRetries();
            iSame = link.getSame();
            sDir = link.getDir();
//							if (!sDir.equals(""))
//							{
            sDir="\""+sDir+"\"";
//							}
            iPriority = link.getPriority();
            sMessage ="GET "+"\""+sUrl+"\" "+iRetries+" "+iPriority+" "+sDir+" "+iSame;
          }
          // PROXY ?
          //AddProxyInfo(sMessage);
          m_bBusy = true;
          Send(sMessage);
          break;
        }
        else
          m_iActual++;
      }
      if(m_iActual >= m_userdat.numberofLinks())
      {
        //			System.out.println("Massensendung fertig");
        m_bTransmitting = false;
        m_bBusy = false;
        m_master.setBusy(false);
      }
    }
  }


  /** Updatet die Informationen aller gesendeten Links - kann nur von Receive aufgerufen werden */
  void SendUpdate()
  {
    //	System.out.println("->SendUpdate");
    if(m_iUpdate >= m_links_to_update.length || m_links_to_update[m_iUpdate]==-1)
    {
//			System.out.println("Massenupdate fertig");
      m_iUpdate = 0;
      m_bTransmitting = false;
      m_bBusy = false;
      m_master.setBusy(false);
      return;
    }
    int iType;
    String sServer,sUser,sPwd,sMessage;
    if(m_bConnected || m_userdat.isProxy())
    {
      // Link bertragen
      sMessage = "STATUS "+m_links_to_update[m_iUpdate];
      // PROXY ?
      //AddProxyInfo(sMessage);
      m_bBusy = true;
      Send(sMessage);
    }
  }


/*
         void SendUpdate()
 {
  //	System.out.println("->SendUpdate");

         int iType;
         String sServer,sUser,sPwd,sMessage;
         if(m_bConnected || m_userdat.isProxy())
         {
  // Link bertragen
                 while(m_iUpdate < m_userdat.numberofLinks())
                 {
                         Link link = m_userdat.getLink(m_iUpdate);
                         iType = link.getType();

                         if(iType==-1)// && (link.getState().equals("downloading") || link.getState().equals("queing")))
                         {

                                 sMessage = "STATUS "+link.getID();
  // PROXY ?
  //AddProxyInfo(sMessage);
                                 m_bBusy = true;
                                 Send(sMessage);
                                 break;
                         }
                         else
                                 m_iUpdate++;
                 }
                 if(m_iUpdate >= m_userdat.numberofLinks())
                 {
  //			System.out.println("Massenupdate fertig");
                         m_bTransmitting = false;
                         m_bBusy = false;
                 }
         }
 }
*/




  // Parametergewinnung - Vector[0] = false|true ; Vector[1] = Parameter
  // *******************************************************************
  /** Liefert die Parameter aus einem String
   *	Rckgabewert: fals Parameter vorhanden -> [true,Parameter]
   *				  sonst					   -> [false, ]
   */
  public Vector getParameter(int which,String separator,String source)
  {
    int index,end;
    Vector result = new Vector();
    String parameter;
    index = 0;
    end = 0;
    try
    {
      for(int i=0;i<=which;i++)
      {
        index = end;
        if (index>=source.length() || index<0)
        {
          // Nichts gefunden
          result.add(new Boolean(false));
          result.add("");
          return result;
        }

        if (source.charAt(index+1) =='"')
        {
          index++;
          end = source.indexOf("\"",index+1)+1;
        }
        else
        {
          end=source.indexOf(separator,index+1);
        }
      }
      if(end<0)
        parameter = source.substring(index).trim();
      else
        parameter = source.substring(index,end).trim();
      if(parameter.charAt(0)=='"')
        parameter = parameter.substring(1);
      if(parameter.charAt(parameter.length()-1)=='"')
        parameter = parameter.substring(0,parameter.length()-1);
      result.add(new Boolean(true));
      result.add(parameter);
      return result;
    }
    catch(Exception E)
    {
//			System.out.println("Error in getParameter: "+E.toString());
      result.add(new Boolean(false));
      result.add("");
      return result;
    }
  }


  // Daten von/an UserDat weiterleiten
  // *********************************

  /** Fgt einen neuen Link in UserDat ein
 @param newlink einzufgender Link
 */
  public void InsertLink(Link newlink)
  {
    m_userdat.addLink(newlink);
  }

  /** Fgt einen neuen Link in UserDat ein
 @param newlink einzufgender URL
 */
  public void InsertLink(String newlink)
  {
    m_userdat.addLink(newlink);
  }

  /** Liefert eine Enumeration aller gespeicherten Links
   */
  public java.util.Enumeration getLinks()
  {
    return m_userdat.getLinks();
  }

  /** Liefert eine Reprsentation aller gespeicherten Links (darstellen der Links in einer Tabelle)
   */
  public Object[][] showLinks()
  {
    return m_userdat.showLinks();
  }

  /** Setzt die Server Informationen
 @param server Serveradresse
 @param port Portnummer
 @param user Username
 @param pwd Pawort
 @param proxy Legt fest ob Kommunikation ber einen Proxyserver laufen soll (Proxyk. noch nicht implementiert)
 @param proxyserver Proxy Serveradresse
 @param proxyport Proxy Portnummer
 */
  public void setServer(String server,int port,String user,String pwd,boolean proxy,String proxyserver,int proxyport)
  {
    m_userdat.setServerInfo(server, port, user, pwd, proxy, proxyserver, proxyport);
    m_bConnected = false;
  }

  /** Setzt Download Inforamtionen
 @param dir Relatives Verzeichnis in das der Download gespeichert werden soll
 @param retries Anzahl der Downloadversuche
 @param same Gibt an wieviele Downloads von einem Server zur gleichen Zeit gestartet werden sollen
 @param priority Legt die Prioritt des Downloads fest
 */
  public void setDownload(String dir,int retries,int same,int priority)
  {
    m_userdat.setDownloadInfo(dir,retries,same,priority);
  }


  /** Liefert den aktuell eingestellten Namen des Downloadverzeichnis zurck
   */
  public String getDir()
  {
    return m_userdat.getDir();
  }

  /** Liefert die aktuell eingestellte Anzahl der gleichzeitigen Downloads von einem Server
   */
  public int getSame()
  {
    return m_userdat.getSame();
  }

  /** Liefert die aktuell eingestellte Anzahl der Downloadversuche
   */
  public int getRetries()
  {
    return m_userdat.getRetries();
  }

  /** Liefert die aktuell eingestellte Prioritt
   */
  public int getPriority()
  {
    return m_userdat.getPriority();
  }

  /** Liefert den Usernamen
   */
  public String getUser()
  {
    return m_userdat.getUser();
  }

  /** Liefert das Pawort
   */
  public String getPwd()
  {
    return m_userdat.getPwd();
  }

  /** Liefert die Serveradresse
   */
  public String getServer()
  {
    return m_userdat.getServer();
  }

  /** Liefert den Serverport
   */
  public int getPort()
  {
    return m_userdat.getPort();
  }

  /** berprft ob Kommunikation ber einen Proxyserver erfolgen soll
   */
  public boolean isProxy()
  {
    return m_userdat.isProxy();
  }

  /** Liefert die Adresse des eingestellten Proxyservers
   */
  public String getProxyServer()
  {
    return m_userdat.getProxyServer();
  }

  /** Liefert die Portnummer des eingestellen Proxyservers
   */
  public int getProxyPort()
  {
    return m_userdat.getProxyPort();
  }

  /** Liefert den als Telnetclient spezifizierten Programmpfad
   */
  public String getTelnet()
  {
    return m_userdat.getTelnet();
  }

  /** Liefert den als Fftpclient spezifizierten Programmpfad
   */
  public String getFtp()
  {
    return m_userdat.getFtp();
  }

  /** Spezifiziert Programmpfade fr Telnetclient und Ftpclient
   */
  public void setPrograms(String ftp,String telnet)
  {
    m_userdat.setPrograms(ftp,telnet);
  }

  /** Importiert die gespeicherten Links aus einer Datei
 @param mode ?
 @param which ?
 @param filename Dateiname
 */
  public boolean File2Links(int mode, int which,String filename,String directory)
  {
    return m_userdat.File2Links(mode,which,filename,directory);
  }


  /** Exportiert die gespeicherten Links in eine Datei
 @param mode ?
 @param which ?
 @param filename Dateiname
 */
  public boolean Links2File(int mode, int which,String filename)
  {
    return m_userdat.Links2File(mode,which,filename);
  }


  // UserDat in File speichern
  // *************************

  /** Liest Userdat aus Datei ein
   */
  public boolean ReadUserData()
  {
    try
    {
      if (m_bIOrights)
      {
        FileInputStream filestream = new FileInputStream(new File(m_sSavefile));
        ObjectInputStream in = new ObjectInputStream(filestream);
        m_userdat = (UserDat) in.readObject();
        in.close();
        filestream.close();
        return true;
      }
      else
        return false;
    }
    catch(Exception e)
    {
      //System.out.println(e.toString());
      return false;
    }
  }


  // UserDat aus File lesen
  // **********************

  /** Speichert Userdat in eine Datei
   */
  public boolean SaveUserData()
  {
    try
    {
      if (m_bIOrights)
      {
//				System.out.println("Schreiben des Sicherungsfiles");
        FileOutputStream filestream = new FileOutputStream(new File(m_sSavefile));
        ObjectOutputStream out = new ObjectOutputStream(filestream);
        out.writeObject(m_userdat);
        out.close();
        filestream.close();
//				System.out.println(m_sSavefile+" erfolgreich gesichert");
        return true;
      }
      else
        return false;
    }
    catch(Exception e)
    {
//			System.out.println(e.toString());
      return false;
    }
  }



// *************
// *** ADMIN ***
// *************
  public void addUser(String username,String passwd,String adminUser)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(username);
        parameter.add(passwd);
        parameter.add(new Boolean(adminUser));
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_ADDUSER);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1000,"ADDUSER");
    }
  }

  public void delUser(String username)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(username);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_DELUSER);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1001,"DELUSER");
    }
  }

  public void getLinks(String username)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(username);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_GETLINKS);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1002,"GETDOWNLAODS");
    }
  }

  public void delDownload(String downloadid)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(new Integer(downloadid));
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_DELDOWNLOAD);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1003,"DELDOWNLAOD");
    }
  }

  public void delDownload(Vector downloadid)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.addAll(downloadid);
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_DELDOWNLOAD);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1003,"DELDOWNLAOD");
    }
  }

  public void setMaxDownloads(String number)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(new Integer(number));
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_SETMAXDOWNLOADS);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1004,"SETMAXDOWNLAODS");
    }
  }

  public void getMaxDownloads()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_GETMAXDOWNLOADS);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1005,"GETMAXDOWNLAODS");
    }
  }


  public void getUsers()
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        DaunyMessage newMessage = new DaunyMessage(null);
        newMessage.setType(newMessage.MSG_GETUSERS);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1006,"GETUSERS");
    }
  }

  public void userEnable(String username,String enable)
  {
    if (isSendAble())
    {
      if (m_bCSend)
      {
        Vector parameter = new Vector();
        parameter.add(username);
        parameter.add(new Boolean(enable));
        DaunyMessage newMessage = new DaunyMessage(parameter);
        newMessage.setType(newMessage.MSG_USERENABLE);
        Send(newMessage);
      }
      else
      {

      }
      m_master.UpdateSend(1007,"USERENABLE");
    }
  }

  // Empfangen

  /** Funktion AddUser
 @param message AddUser Nachricht
 gibt 0 weiter wenn die Aktion erfolgreich war ansonsten den Errorcode
 */
  void Receive_AddUser(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1000,new Integer(0));
    }
    else
    {
      m_master.setOutput(1000,(Integer) message.getParameters().get(0));
    }
  }

  /** Funktion DelUser
  @param message DelUser Nachricht
  gibt 0 weiter wenn die Aktion erfolgreich war ansonsten den Errorcode
  */
  void Receive_DelUser(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1001,new Integer(0));
    }
    else
    {
      m_master.setOutput(1001,(Integer) message.getParameters().get(0));
    }
  }

  /** Funktion GetDownloads
  @param message GetDownloads Nachricht
  */
  void Receive_GetDownloads(DaunyMessage message)
  {
    Vector links=null;
    if (message.getSuxcess())
    {
      links = message.getParameters();
    }

    m_master.setOutput(1002,links);
  }

  /** Funktion GetDownloads
  @param message DelDownloads Nachricht
  */
  void Receive_DelDownload(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1003,new Integer(0));
    }
    else
    {
      m_master.setOutput(1003,(Integer) message.getParameters().get(0));
    }
  }

  void Receive_SetMaxDownloads(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1004,new Integer(0));
    }
    else
    {
      m_master.setOutput(1004,new Integer(1));
    }
  }


  void Receive_GetMaxDownloads(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1005,(Integer) message.getParameters().get(0));
    }
    else
    {
      m_master.setOutput(1005,new Integer(-1));
    }
  }

  void Receive_GetUsers(DaunyMessage message)
  {
//NOCH TESTEN
    Vector users=null;
    if (message.getSuxcess())
    {
      users = message.getParameters();
    }

    m_master.setOutput(1006,users);
  }

  void Receive_UserEnable(com.jdauny.DaunyMessage message)
  {
    if(message.getSuxcess())
    {
      m_master.setOutput(1007,new Integer(1007));
    }
    else
    {
      m_master.setOutput(1007,(Integer) message.getParameters().get(0));
    }
  }
// ******************
// *** ADMIN ENDE ***
// ******************

  public static void main(String args[])
  {
//		DaunySocket test = new DaunySocket();
//		try
//		{
//		String message = "STATUS 13 \"KAI\" \"http://www.jet2web.net/jet2web/FE/LayoutTemplates/FE_Layout/0,2840,6-1-186458-0,00.html\" 0 \"finished\" \"Invalid argument\"";
//		System.out.println(message);
//		System.out.println(test.getParameter(1," ",message));
//		System.out.println(test.getParameter(4," ",message));
//		System.out.println(test.getParameter(5," ",message));
//		System.out.println(test.getParameter(6," ",message));
//		}
//		catch(Exception E)
//		{
//			System.out.println("Error: "+E.toString());
//		}
//		test.setConnectData("subzklnx.tu-graz.ac.at",5561,"kai","fritz");
/*		if (test.Connect())
  {
   Receiver testR = new Receiver();
   testR.setMaster(test,test.m_IN);
   testR.start();
   test.Login();
   test.Send("VAR");
   test.Send("VER");

  }
*/
  }
}
